from __future__ import absolute_import, division, print_function, unicode_literals


def list_secrets(secrets_table):
    response = secrets_table.scan(
        ProjectionExpression="#N, version",
        ExpressionAttributeNames={"#N": "name"}
    )
    return response["Items"]
